@@ -27,11 +27,13 @@ T = TypeVar("T", bound=Callable)
 
 def check_authentication() -> None:
     """Checks that the request has valid authorization information."""
-    response = current_app.api_auth.requires_authentication(Response)()
-    if response.status_code != 200:
-        # since this handler only checks authentication, not authorization,
-        # we should always return 401
-        raise Unauthenticated(headers=response.headers)
+    for auth in current_app.api_auth:
+        response = auth.requires_authentication(Response)()
+        if response.status_code == 200:
+            return
+    # since this handler only checks authentication, not authorization,
+    # we should always return 401
+    raise Unauthenticated(headers=response.headers)
 
 
 def requires_access(permissions: Optional[Sequence[Tuple[str, str]]] = None) -> Callable[[T], T]:
