@@ -40,7 +40,7 @@ from airflow.utils import cli as cli_utils
 from airflow.utils.cli import setup_locations, setup_logging
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.process_utils import check_if_pidfile_process_is_running
-from airflow.www.app import cached_app, create_app
+from airflow.www.app import create_app
 
 log = logging.getLogger(__name__)
 
@@ -133,7 +133,7 @@ class GunicornMonitor(LoggingMixin):
         def ready_prefix_on_cmdline(proc):
             try:
                 cmdline = proc.cmdline()
-                if len(cmdline) > 0:  # pylint: disable=len-as-condition
+                if len(cmdline) > 0:
                     return settings.GUNICORN_WORKER_READY_PREFIX in cmdline[0]
             except psutil.NoSuchProcess:
                 pass
@@ -201,7 +201,7 @@ class GunicornMonitor(LoggingMixin):
 
     def start(self) -> NoReturn:
         """Starts monitoring the webserver."""
-        try:  # pylint: disable=too-many-nested-blocks
+        try:
             self._wait_until_true(
                 lambda: self.num_workers_expected == self._get_num_workers_running(),
                 timeout=self.master_timeout,
@@ -313,7 +313,7 @@ class GunicornMonitor(LoggingMixin):
                 self._reload_gunicorn()
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli
 def webserver(args):
     """Starts Airflow Webserver"""
     print(settings.HEADER)
@@ -354,11 +354,6 @@ def webserver(args):
             ssl_context=(ssl_cert, ssl_key) if ssl_cert and ssl_key else None,
         )
     else:
-        # This pre-warms the cache, and makes possible errors
-        # get reported earlier (i.e. before demonization)
-        os.environ['SKIP_DAGS_PARSING'] = 'True'
-        app = cached_app(None)
-        os.environ.pop('SKIP_DAGS_PARSING')
 
         pid_file, stdout, stderr, log_file = setup_locations(
             "webserver", args.pid, args.stdout, args.stderr, args.log_file
@@ -369,28 +364,20 @@ def webserver(args):
 
         print(
             textwrap.dedent(
-                '''\
+                f'''\
                 Running the Gunicorn Server with:
-                Workers: {num_workers} {workerclass}
-                Host: {hostname}:{port}
+                Workers: {num_workers} {args.workerclass}
+                Host: {args.hostname}:{args.port}
                 Timeout: {worker_timeout}
                 Logfiles: {access_logfile} {error_logfile}
                 Access Logformat: {access_logformat}
-                =================================================================\
-            '''.format(
-                    num_workers=num_workers,
-                    workerclass=args.workerclass,
-                    hostname=args.hostname,
-                    port=args.port,
-                    worker_timeout=worker_timeout,
-                    access_logfile=access_logfile,
-                    error_logfile=error_logfile,
-                    access_logformat=access_logformat,
-                )
+                ================================================================='''
             )
         )
 
         run_args = [
+            sys.executable,
+            '-m',
             'gunicorn',
             '--workers',
             str(num_workers),
@@ -427,7 +414,7 @@ def webserver(args):
 
         gunicorn_master_proc = None
 
-        def kill_proc(signum, _):  # pylint: disable=unused-argument
+        def kill_proc(signum, _):
             log.info("Received signal: %s. Closing gunicorn.", signum)
             gunicorn_master_proc.terminate()
             with suppress(TimeoutError):
@@ -454,6 +441,11 @@ def webserver(args):
             ).start()
 
         if args.daemon:
+            # This makes possible errors get reported before daemonization
+            os.environ['SKIP_DAGS_PARSING'] = 'True'
+            app = create_app(None)
+            os.environ.pop('SKIP_DAGS_PARSING')
+
             handle = setup_logging(log_file)
 
             base, ext = os.path.splitext(pid_file)
