@@ -94,11 +94,8 @@ class PodGenerator:
     the first container in the list of containers.
 
     :param pod: The fully specified pod. Mutually exclusive with `path_or_string`
-    :type pod: Optional[kubernetes.client.models.V1Pod]
     :param pod_template_file: Path to YAML file. Mutually exclusive with `pod`
-    :type pod_template_file: Optional[str]
     :param extract_xcom: Whether to bring up a container for xcom
-    :type extract_xcom: bool
     """
 
     def __init__(
@@ -225,9 +222,7 @@ class PodGenerator:
         """
         :param base_pod: has the base attributes which are overwritten if they exist
             in the client pod and remain if they do not exist in the client_pod
-        :type base_pod: k8s.V1Pod
         :param client_pod: the pod that the client wants to create.
-        :type client_pod: k8s.V1Pod
         :return: the merged pods
 
         This can't be done recursively as certain fields some overwritten, and some concatenated.
@@ -248,9 +243,7 @@ class PodGenerator:
         Merge kubernetes Metadata objects
         :param base_meta: has the base attributes which are overwritten if they exist
             in the client_meta and remain if they do not exist in the client_meta
-        :type base_meta: k8s.V1ObjectMeta
         :param client_meta: the spec that the client wants to create.
-        :type client_meta: k8s.V1ObjectMeta
         :return: the merged specs
         """
         if base_meta and not client_meta:
@@ -274,9 +267,7 @@ class PodGenerator:
         """
         :param base_spec: has the base attributes which are overwritten if they exist
             in the client_spec and remain if they do not exist in the client_spec
-        :type base_spec: k8s.V1PodSpec
         :param client_spec: the spec that the client wants to create.
-        :type client_spec: k8s.V1PodSpec
         :return: the merged specs
         """
         if base_spec and not client_spec:
@@ -300,9 +291,7 @@ class PodGenerator:
         """
         :param base_containers: has the base attributes which are overwritten if they exist
             in the client_containers and remain if they do not exist in the client_containers
-        :type base_containers: List[k8s.V1Container]
         :param client_containers: the containers that the client wants to create.
-        :type client_containers: List[k8s.V1Container]
         :return: the merged containers
 
         The runs recursively over the list of containers.
@@ -337,8 +326,9 @@ class PodGenerator:
         pod_override_object: Optional[k8s.V1Pod],
         base_worker_pod: k8s.V1Pod,
         namespace: str,
-        scheduler_job_id: int,
+        scheduler_job_id: str,
         run_id: Optional[str] = None,
+        map_index: int = -1,
     ) -> k8s.V1Pod:
         """
         Construct a pod by gathering and consolidating the configuration from 3 places:
@@ -359,13 +349,16 @@ class PodGenerator:
             'try_number': str(try_number),
         }
         labels = {
-            'airflow-worker': make_safe_label_value(str(scheduler_job_id)),
+            'airflow-worker': make_safe_label_value(scheduler_job_id),
             'dag_id': make_safe_label_value(dag_id),
             'task_id': make_safe_label_value(task_id),
             'try_number': str(try_number),
             'airflow_version': airflow_version.replace('+', '-'),
             'kubernetes_executor': 'True',
         }
+        if map_index >= 0:
+            annotations['map_index'] = str(map_index)
+            labels['map_index'] = str(map_index)
         if date:
             annotations['execution_date'] = date.isoformat()
             labels['execution_date'] = datetime_to_label_safe_datestring(date)
@@ -440,7 +433,7 @@ class PodGenerator:
         return api_client._ApiClient__deserialize_model(pod_dict, k8s.V1Pod)
 
     @staticmethod
-    def make_unique_pod_id(pod_id: str) -> str:
+    def make_unique_pod_id(pod_id: str) -> Optional[str]:
         r"""
         Kubernetes pod names must consist of one or more lowercase
         rfc1035/rfc1123 labels separated by '.' with a maximum length of 253
@@ -459,11 +452,14 @@ class PodGenerator:
             return None
 
         safe_uuid = uuid.uuid4().hex  # safe uuid will always be less than 63 chars
-        # Strip trailing '-' and '.' as they can't be followed by '.'
-        trimmed_pod_id = pod_id[:MAX_LABEL_LEN].rstrip('-.')
 
-        safe_pod_id = f"{trimmed_pod_id}.{safe_uuid}"
-        return safe_pod_id
+        # Get prefix length after subtracting the uuid length. Clean up '.' and '-' from
+        # end of podID ('.' can't be followed by '-').
+        label_prefix_length = MAX_LABEL_LEN - len(safe_uuid) - 1  # -1 for separator
+        trimmed_pod_id = pod_id[:label_prefix_length].rstrip('-.')
+
+        # previously used a '.' as the separator, but this could create errors in some situations
+        return f"{trimmed_pod_id}-{safe_uuid}"
 
 
 def merge_objects(base_obj, client_obj):
@@ -501,7 +497,6 @@ def extend_object_field(base_obj, client_obj, field_name):
     :param client_obj: an object which has a property `field_name` that is a list.
         A copy of this object is returned with `field_name` modified
     :param field_name: the name of the list field
-    :type field_name: str
     :return: the client_obj with the property `field_name` being the two properties appended
     """
     client_obj_cp = copy.deepcopy(client_obj)
