@@ -19,8 +19,10 @@
 import os
 from subprocess import PIPE, STDOUT, Popen
 from tempfile import NamedTemporaryFile, TemporaryDirectory, gettempdir
+from typing import Sequence
 
 from airflow.sensors.base import BaseSensorOperator
+from airflow.utils.context import Context
 
 
 class BashSensor(BaseSensorOperator):
@@ -30,18 +32,15 @@ class BashSensor(BaseSensorOperator):
 
     :param bash_command: The command, set of commands or reference to a
         bash script (must be '.sh') to be executed.
-    :type bash_command: str
 
     :param env: If env is not None, it must be a mapping that defines the
         environment variables for the new process; these are used instead
         of inheriting the current process environment, which is the default
         behavior. (templated)
-    :type env: dict
     :param output_encoding: output encoding of bash command.
-    :type output_encoding: str
     """
 
-    template_fields = ('bash_command', 'env')
+    template_fields: Sequence[str] = ('bash_command', 'env')
 
     def __init__(self, *, bash_command, env=None, output_encoding='utf-8', **kwargs):
         super().__init__(**kwargs)
@@ -49,7 +48,7 @@ class BashSensor(BaseSensorOperator):
         self.env = env
         self.output_encoding = output_encoding
 
-    def poke(self, context):
+    def poke(self, context: Context):
         """
         Execute the bash command in a temporary directory
         which will be cleaned afterwards
@@ -74,10 +73,10 @@ class BashSensor(BaseSensorOperator):
                     env=self.env,
                     preexec_fn=os.setsid,
                 ) as resp:
-                    self.log.info("Output:")
-                    for line in iter(resp.stdout.readline, b''):
-                        line = line.decode(self.output_encoding).strip()
-                        self.log.info(line)
+                    if resp.stdout:
+                        self.log.info("Output:")
+                        for line in iter(resp.stdout.readline, b''):
+                            self.log.info(line.decode(self.output_encoding).strip())
                     resp.wait()
                     self.log.info("Command exited with return code %s", resp.returncode)
 
