@@ -32,6 +32,5 @@ class DagrunRunningDep(BaseTIDep):
         dr = ti.get_dagrun(session)
         if dr.state != State.RUNNING:
             yield self._failing_status(
-                reason="Task instance's dagrun was not in the 'running' state but in "
-                "the state '{}'.".format(dr.state)
+                reason=f"Task instance's dagrun was not in the 'running' state but in the state '{dr.state}'."
             )
