@@ -17,24 +17,33 @@
 # under the License.
 import copy
 import re
+import signal
 import warnings
 from datetime import datetime
 from functools import reduce
 from itertools import filterfalse, tee
-from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, Iterable, List, Optional, Tuple, TypeVar
-from urllib import parse
-
-import flask
-import jinja2
-import jinja2.nativetypes
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterable,
+    List,
+    MutableMapping,
+    Optional,
+    Tuple,
+    TypeVar,
+)
 
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException
-from airflow.utils.context import Context
 from airflow.utils.module_loading import import_string
 
 if TYPE_CHECKING:
-    from airflow.models import TaskInstance
+    import jinja2
+
+    from airflow.models.taskinstance import TaskInstance
 
 KEY_REGEX = re.compile(r'^[\w.-]+$')
 GROUP_KEY_REGEX = re.compile(r'^[\w-]+$')
@@ -52,8 +61,8 @@ def validate_key(k: str, max_length: int = 250):
         raise AirflowException(f"The key has to be less than {max_length} characters")
     if not KEY_REGEX.match(k):
         raise AirflowException(
-            "The key ({k}) has to be made of alphanumeric characters, dashes, "
-            "dots and underscores exclusively".format(k=k)
+            f"The key {k!r} has to be made of alphanumeric characters, dashes, "
+            f"dots and underscores exclusively"
         )
 
 
@@ -65,7 +74,7 @@ def validate_group_key(k: str, max_length: int = 200):
         raise AirflowException(f"The key has to be less than {max_length} characters")
     if not GROUP_KEY_REGEX.match(k):
         raise AirflowException(
-            f"The key ({k}) has to be made of alphanumeric characters, dashes and underscores exclusively"
+            f"The key {k!r} has to be made of alphanumeric characters, dashes and underscores exclusively"
         )
 
 
@@ -82,21 +91,35 @@ def alchemy_to_dict(obj: Any) -> Optional[Dict]:
     return output
 
 
-def ask_yesno(question: str) -> bool:
-    """Helper to get yes / no answer from user."""
+def ask_yesno(question: str, default: Optional[bool] = None) -> bool:
+    """Helper to get a yes or no answer from the user."""
     yes = {'yes', 'y'}
     no = {'no', 'n'}
 
-    done = False
     print(question)
-    while not done:
+    while True:
         choice = input().lower()
+        if choice == "" and default is not None:
+            return default
         if choice in yes:
             return True
-        elif choice in no:
+        if choice in no:
             return False
-        else:
-            print("Please respond by yes or no.")
+        print("Please respond with y/yes or n/no.")
+
+
+def prompt_with_timeout(question: str, timeout: int, default: Optional[bool] = None) -> bool:
+    """Ask the user a question and timeout if they don't respond"""
+
+    def handler(signum, frame):
+        raise AirflowException(f"Timeout {timeout}s reached")
+
+    signal.signal(signal.SIGALRM, handler)
+    signal.alarm(timeout)
+    try:
+        return ask_yesno(question, default)
+    finally:
+        signal.alarm(0)
 
 
 def is_container(obj: Any) -> bool:
@@ -145,8 +168,10 @@ def as_flattened_list(iterable: Iterable[Iterable[T]]) -> List[T]:
     return [e for i in iterable for e in i]
 
 
-def parse_template_string(template_string):
+def parse_template_string(template_string: str) -> Tuple[Optional[str], Optional["jinja2.Template"]]:
     """Parses Jinja template string."""
+    import jinja2
+
     if "{{" in template_string:  # jinja mode
         return None, jinja2.Template(template_string)
     else:
@@ -229,14 +254,15 @@ def build_airflow_url_with_query(query: Dict[str, Any]) -> str:
     For example:
     'http://0.0.0.0:8000/base/graph?dag_id=my-task&root=&execution_date=2020-10-27T10%3A59%3A25.615587
     """
-    view = conf.get('webserver', 'dag_default_view').lower()
-    url = flask.url_for(f"Airflow.{view}")
-    return f"{url}?{parse.urlencode(query)}"
+    import flask
+
+    view = conf.get_mandatory_value('webserver', 'dag_default_view').lower()
+    return flask.url_for(f"Airflow.{view}", **query)
 
 
 # The 'template' argument is typed as Any because the jinja2.Template is too
 # dynamic to be effectively type-checked.
-def render_template(template: Any, context: Context, *, native: bool) -> Any:
+def render_template(template: Any, context: MutableMapping[str, Any], *, native: bool) -> Any:
     """Render a Jinja2 template with given Airflow context.
 
     The default implementation of ``jinja2.Template.render()`` converts the
@@ -259,15 +285,75 @@ def render_template(template: Any, context: Context, *, native: bool) -> Any:
     except Exception:
         env.handle_exception()  # Rewrite traceback to point to the template.
     if native:
+        import jinja2.nativetypes
+
         return jinja2.nativetypes.native_concat(nodes)
     return "".join(nodes)
 
 
-def render_template_to_string(template: jinja2.Template, context: Context) -> str:
+def render_template_to_string(template: "jinja2.Template", context: MutableMapping[str, Any]) -> str:
     """Shorthand to ``render_template(native=False)`` with better typing support."""
     return render_template(template, context, native=False)
 
 
-def render_template_as_native(template: jinja2.Template, context: Context) -> Any:
+def render_template_as_native(template: "jinja2.Template", context: MutableMapping[str, Any]) -> Any:
     """Shorthand to ``render_template(native=True)`` with better typing support."""
     return render_template(template, context, native=True)
+
+
+def exactly_one(*args) -> bool:
+    """
+    Returns True if exactly one of *args is "truthy", and False otherwise.
+
+    If user supplies an iterable, we raise ValueError and force them to unpack.
+    """
+    if is_container(args[0]):
+        raise ValueError(
+            "Not supported for iterable args. Use `*` to unpack your iterable in the function call."
+        )
+    return sum(map(bool, args)) == 1
+
+
+def prune_dict(val: Any, mode='strict'):
+    """
+    Given dict ``val``, returns new dict based on ``val`` with all
+    empty elements removed.
+
+    What constitutes "empty" is controlled by the ``mode`` parameter.  If mode is 'strict'
+    then only ``None`` elements will be removed.  If mode is ``truthy``, then element ``x``
+    will be removed if ``bool(x) is False``.
+    """
+
+    def is_empty(x):
+        if mode == 'strict':
+            return x is None
+        elif mode == 'truthy':
+            return bool(x) is False
+        raise ValueError("allowable values for `mode` include 'truthy' and 'strict'")
+
+    if isinstance(val, dict):
+        new_dict = {}
+        for k, v in val.items():
+            if is_empty(v):
+                continue
+            elif isinstance(v, (list, dict)):
+                new_val = prune_dict(v, mode=mode)
+                if new_val:
+                    new_dict[k] = new_val
+            else:
+                new_dict[k] = v
+        return new_dict
+    elif isinstance(val, list):
+        new_list = []
+        for v in val:
+            if is_empty(v):
+                continue
+            elif isinstance(v, (list, dict)):
+                new_val = prune_dict(v, mode=mode)
+                if new_val:
+                    new_list.append(new_val)
+            else:
+                new_list.append(v)
+        return new_list
+    else:
+        return val
