@@ -14,18 +14,19 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 import contextlib
 from functools import wraps
 from inspect import signature
-from typing import Callable, TypeVar
+from typing import Callable, Iterator, TypeVar, cast
 
 from airflow import settings
 
 
 @contextlib.contextmanager
-def create_session():
+def create_session() -> Iterator[settings.SASession]:
     """Contextmanager that will create and teardown a session."""
+    if not settings.Session:
+        raise RuntimeError("Session must be set before!")
     session = settings.Session()
     try:
         yield session
@@ -37,7 +38,7 @@ def create_session():
         session.close()
 
 
-RT = TypeVar("RT")  # pylint: disable=invalid-name
+RT = TypeVar("RT")
 
 
 def find_session_idx(func: Callable[..., RT]) -> int:
@@ -70,3 +71,10 @@ def provide_session(func: Callable[..., RT]) -> Callable[..., RT]:
                 return func(*args, session=session, **kwargs)
 
     return wrapper
+
+
+# A fake session to use in functions decorated by provide_session. This allows
+# the 'session' argument to be of type Session instead of Optional[Session],
+# making it easier to type hint the function body without dealing with the None
+# case that can never happen at runtime.
+NEW_SESSION: settings.SASession = cast(settings.SASession, None)
